//===================================RISCKER====================================
//==================================Operations==================================
//------------------------------------------------------------------------------
//File Name:	operations.v
//Purpose:		Main operations module that contains ALU, shifters, Multiplier and divider
//Parameters:	None
//------------------------------------------------------------------------------
//=====================Alfredo Gualdron - Jose P. Pinilla=======================
//=================================Semillero ADT================================
//================================UPB Bucaramanga===============================
//Website:	http://semilleroadt.upbbga.edu.co/xiscker
//E-mail:	semilleroadt@gmail.com
//------------------------------------------------------------------------------
module operations (a,b,operation,zero,sign,cout,y);

input [15:0]a,b;
input [10:0]operation;
output zero,sign,cout;
output reg[15:0]y;

//Decode operation
wire [2:0] aluop = operation[10:8];
wire [3:0] shamt = operation[7:4];
wire [1:0] shift_mode = operation[3:2];
wire [1:0] select = operation[1:0]; //0-ALU, 1-Shift, 2-Mult, 3-Div

//Modules Outputs
wire [15:0]alu_result;

wire [15:0]shift_result;

wire [15:0]mult_result;

wire [15:0] div_result;

//Operation Modules
alu alu_inst (
    .a(a), 
    .b(b), 
    .aluop(aluop), 
    .y(alu_result), 
    .zero(zero), 
    .cout(cout),
	.sign(sign)	
    );
	defparam alu_inst.N = 16;

//Logic Shifts Left/Right, Rotate and Arithmetic Shift Right
shifter #(15,4)shifter_inst (
    .a(b), 
    .shamt(shamt), 
    .shift_mode(shift_mode), 
    .y(shift_result)
    );

//Multiplier Primitive
//MULT18X18 MULT18X18_inst (
//      .P(mult_result),    		// 36-bit multiplier output
//      .A({10'd0,a[7:0]}),    	// 18-bit multiplier input
//      .B({10'd0,b[7:0]})     	// 18-bit multiplier input
//   );

assign mult_result = a * b;

//Hardware Divider
divider #(8)divider_AF (
    .dividend(a[7:0]), 
    .divisor(b[7:0]), 
    .result(div_result)
    );

//Outputs Multiplexer
always @ *
case (select)
	2'b00:	y <= alu_result;
	2'b01:	y <= shift_result;
	2'b10:	y <= mult_result;
	2'b11:	y <= div_result;
endcase

endmodule
